//
//  Copyright (c) 2021 Open Whisper Systems. All rights reserved.
//

import Foundation
import GRDB
import SignalCoreKit

// NOTE: This file is generated by /Scripts/sds_codegen/sds_generate.py.
// Do not manually edit it, instead run `sds_codegen.sh`.

// MARK: - Record

public struct JobRecordRecord: SDSRecord {
    public weak var delegate: SDSRecordDelegate?

    public var tableMetadata: SDSTableMetadata {
        return SSKJobRecordSerializer.table
    }

    public static let databaseTableName: String = SSKJobRecordSerializer.table.tableName

    public var id: Int64?

    // This defines all of the columns used in the table
    // where this model (and any subclasses) are persisted.
    public let recordType: SDSRecordType
    public let uniqueId: String

    // Properties
    public let failureCount: UInt
    public let label: String
    public let status: SSKJobRecordStatus
    public let attachmentIdMap: Data?
    public let contactThreadId: String?
    public let envelopeData: Data?
    public let invisibleMessage: Data?
    public let messageId: String?
    public let removeMessageAfterSending: Bool?
    public let threadId: String?
    public let attachmentId: String?
    public let isMediaMessage: Bool?
    public let serverDeliveryTimestamp: UInt64?

    public enum CodingKeys: String, CodingKey, ColumnExpression, CaseIterable {
        case id
        case recordType
        case uniqueId
        case failureCount
        case label
        case status
        case attachmentIdMap
        case contactThreadId
        case envelopeData
        case invisibleMessage
        case messageId
        case removeMessageAfterSending
        case threadId
        case attachmentId
        case isMediaMessage
        case serverDeliveryTimestamp
    }

    public static func columnName(_ column: JobRecordRecord.CodingKeys, fullyQualified: Bool = false) -> String {
        return fullyQualified ? "\(databaseTableName).\(column.rawValue)" : column.rawValue
    }

    public func didInsert(with rowID: Int64, for column: String?) {
        guard let delegate = delegate else {
            owsFailDebug("Missing delegate.")
            return
        }
        delegate.updateRowId(rowID)
    }
}

// MARK: - Row Initializer

public extension JobRecordRecord {
    static var databaseSelection: [SQLSelectable] {
        return CodingKeys.allCases
    }

    init(row: Row) {
        id = row[0]
        recordType = row[1]
        uniqueId = row[2]
        failureCount = row[3]
        label = row[4]
        status = row[5]
        attachmentIdMap = row[6]
        contactThreadId = row[7]
        envelopeData = row[8]
        invisibleMessage = row[9]
        messageId = row[10]
        removeMessageAfterSending = row[11]
        threadId = row[12]
        attachmentId = row[13]
        isMediaMessage = row[14]
        serverDeliveryTimestamp = row[15]
    }
}

// MARK: - StringInterpolation

public extension String.StringInterpolation {
    mutating func appendInterpolation(jobRecordColumn column: JobRecordRecord.CodingKeys) {
        appendLiteral(JobRecordRecord.columnName(column))
    }
    mutating func appendInterpolation(jobRecordColumnFullyQualified column: JobRecordRecord.CodingKeys) {
        appendLiteral(JobRecordRecord.columnName(column, fullyQualified: true))
    }
}

// MARK: - Deserialization

// TODO: Rework metadata to not include, for example, columns, column indices.
extension SSKJobRecord {
    // This method defines how to deserialize a model, given a
    // database row.  The recordType column is used to determine
    // the corresponding model class.
    class func fromRecord(_ record: JobRecordRecord) throws -> SSKJobRecord {

        guard let recordId = record.id else {
            throw SDSError.invalidValue
        }

        switch record.recordType {
        case .broadcastMediaMessageJobRecord:

            let uniqueId: String = record.uniqueId
            let failureCount: UInt = record.failureCount
            let label: String = record.label
            let sortId: UInt64 = UInt64(recordId)
            let status: SSKJobRecordStatus = record.status
            let attachmentIdMapSerialized: Data? = record.attachmentIdMap
            let attachmentIdMap: [String: [String]] = try SDSDeserialization.unarchive(attachmentIdMapSerialized, name: "attachmentIdMap")

            return OWSBroadcastMediaMessageJobRecord(grdbId: recordId,
                                                     uniqueId: uniqueId,
                                                     failureCount: failureCount,
                                                     label: label,
                                                     sortId: sortId,
                                                     status: status,
                                                     attachmentIdMap: attachmentIdMap)

        case .incomingContactSyncJobRecord:

            let uniqueId: String = record.uniqueId
            let failureCount: UInt = record.failureCount
            let label: String = record.label
            let sortId: UInt64 = UInt64(recordId)
            let status: SSKJobRecordStatus = record.status
            let attachmentId: String = try SDSDeserialization.required(record.attachmentId, name: "attachmentId")

            return OWSIncomingContactSyncJobRecord(grdbId: recordId,
                                                   uniqueId: uniqueId,
                                                   failureCount: failureCount,
                                                   label: label,
                                                   sortId: sortId,
                                                   status: status,
                                                   attachmentId: attachmentId)

        case .incomingGroupSyncJobRecord:

            let uniqueId: String = record.uniqueId
            let failureCount: UInt = record.failureCount
            let label: String = record.label
            let sortId: UInt64 = UInt64(recordId)
            let status: SSKJobRecordStatus = record.status
            let attachmentId: String = try SDSDeserialization.required(record.attachmentId, name: "attachmentId")

            return OWSIncomingGroupSyncJobRecord(grdbId: recordId,
                                                 uniqueId: uniqueId,
                                                 failureCount: failureCount,
                                                 label: label,
                                                 sortId: sortId,
                                                 status: status,
                                                 attachmentId: attachmentId)

        case .sessionResetJobRecord:

            let uniqueId: String = record.uniqueId
            let failureCount: UInt = record.failureCount
            let label: String = record.label
            let sortId: UInt64 = UInt64(recordId)
            let status: SSKJobRecordStatus = record.status
            let contactThreadId: String = try SDSDeserialization.required(record.contactThreadId, name: "contactThreadId")

            return OWSSessionResetJobRecord(grdbId: recordId,
                                            uniqueId: uniqueId,
                                            failureCount: failureCount,
                                            label: label,
                                            sortId: sortId,
                                            status: status,
                                            contactThreadId: contactThreadId)

        case .jobRecord:

            let uniqueId: String = record.uniqueId
            let failureCount: UInt = record.failureCount
            let label: String = record.label
            let sortId: UInt64 = UInt64(recordId)
            let status: SSKJobRecordStatus = record.status

            return SSKJobRecord(grdbId: recordId,
                                uniqueId: uniqueId,
                                failureCount: failureCount,
                                label: label,
                                sortId: sortId,
                                status: status)

        case .messageDecryptJobRecord:

            let uniqueId: String = record.uniqueId
            let failureCount: UInt = record.failureCount
            let label: String = record.label
            let sortId: UInt64 = UInt64(recordId)
            let status: SSKJobRecordStatus = record.status
            let envelopeData: Data? = SDSDeserialization.optionalData(record.envelopeData, name: "envelopeData")
            let serverDeliveryTimestamp: UInt64 = try SDSDeserialization.required(record.serverDeliveryTimestamp, name: "serverDeliveryTimestamp")

            return SSKMessageDecryptJobRecord(grdbId: recordId,
                                              uniqueId: uniqueId,
                                              failureCount: failureCount,
                                              label: label,
                                              sortId: sortId,
                                              status: status,
                                              envelopeData: envelopeData,
                                              serverDeliveryTimestamp: serverDeliveryTimestamp)

        case .messageSenderJobRecord:

            let uniqueId: String = record.uniqueId
            let failureCount: UInt = record.failureCount
            let label: String = record.label
            let sortId: UInt64 = UInt64(recordId)
            let status: SSKJobRecordStatus = record.status
            let invisibleMessageSerialized: Data? = record.invisibleMessage
            let invisibleMessage: TSOutgoingMessage? = try SDSDeserialization.optionalUnarchive(invisibleMessageSerialized, name: "invisibleMessage")
            let isMediaMessage: Bool = try SDSDeserialization.required(record.isMediaMessage, name: "isMediaMessage")
            let messageId: String? = record.messageId
            let removeMessageAfterSending: Bool = try SDSDeserialization.required(record.removeMessageAfterSending, name: "removeMessageAfterSending")
            let threadId: String? = record.threadId

            return SSKMessageSenderJobRecord(grdbId: recordId,
                                             uniqueId: uniqueId,
                                             failureCount: failureCount,
                                             label: label,
                                             sortId: sortId,
                                             status: status,
                                             invisibleMessage: invisibleMessage,
                                             isMediaMessage: isMediaMessage,
                                             messageId: messageId,
                                             removeMessageAfterSending: removeMessageAfterSending,
                                             threadId: threadId)

        default:
            owsFailDebug("Unexpected record type: \(record.recordType)")
            throw SDSError.invalidValue
        }
    }
}

// MARK: - SDSModel

extension SSKJobRecord: SDSModel {
    public var serializer: SDSSerializer {
        // Any subclass can be cast to it's superclass,
        // so the order of this switch statement matters.
        // We need to do a "depth first" search by type.
        switch self {
        case let model as SSKMessageSenderJobRecord:
            assert(type(of: model) == SSKMessageSenderJobRecord.self)
            return SSKMessageSenderJobRecordSerializer(model: model)
        case let model as SSKMessageDecryptJobRecord:
            assert(type(of: model) == SSKMessageDecryptJobRecord.self)
            return SSKMessageDecryptJobRecordSerializer(model: model)
        case let model as OWSSessionResetJobRecord:
            assert(type(of: model) == OWSSessionResetJobRecord.self)
            return OWSSessionResetJobRecordSerializer(model: model)
        case let model as OWSIncomingGroupSyncJobRecord:
            assert(type(of: model) == OWSIncomingGroupSyncJobRecord.self)
            return OWSIncomingGroupSyncJobRecordSerializer(model: model)
        case let model as OWSIncomingContactSyncJobRecord:
            assert(type(of: model) == OWSIncomingContactSyncJobRecord.self)
            return OWSIncomingContactSyncJobRecordSerializer(model: model)
        case let model as OWSBroadcastMediaMessageJobRecord:
            assert(type(of: model) == OWSBroadcastMediaMessageJobRecord.self)
            return OWSBroadcastMediaMessageJobRecordSerializer(model: model)
        default:
            return SSKJobRecordSerializer(model: self)
        }
    }

    public func asRecord() throws -> SDSRecord {
        return try serializer.asRecord()
    }

    public var sdsTableName: String {
        return JobRecordRecord.databaseTableName
    }

    public static var table: SDSTableMetadata {
        return SSKJobRecordSerializer.table
    }
}

// MARK: - DeepCopyable

extension SSKJobRecord: DeepCopyable {

    public func deepCopy() throws -> AnyObject {
        // Any subclass can be cast to it's superclass,
        // so the order of this switch statement matters.
        // We need to do a "depth first" search by type.
        guard let id = self.grdbId?.int64Value else {
            throw OWSAssertionError("Model missing grdbId.")
        }

        if let modelToCopy = self as? SSKMessageSenderJobRecord {
            assert(type(of: modelToCopy) == SSKMessageSenderJobRecord.self)
            let uniqueId: String = modelToCopy.uniqueId
            let failureCount: UInt = modelToCopy.failureCount
            let label: String = modelToCopy.label
            let sortId: UInt64 = modelToCopy.sortId
            let status: SSKJobRecordStatus = modelToCopy.status
            // NOTE: If this generates build errors, you made need to
            // modify DeepCopy.swift to support this type.
            //
            // That might mean:
            //
            // * Implement DeepCopyable for this type (e.g. a model).
            // * Modify DeepCopies.deepCopy() to support this type (e.g. a collection).
            let invisibleMessage: TSOutgoingMessage?
            if let invisibleMessageForCopy = modelToCopy.invisibleMessage {
               invisibleMessage = try DeepCopies.deepCopy(invisibleMessageForCopy)
            } else {
               invisibleMessage = nil
            }
            let isMediaMessage: Bool = modelToCopy.isMediaMessage
            let messageId: String? = modelToCopy.messageId
            let removeMessageAfterSending: Bool = modelToCopy.removeMessageAfterSending
            let threadId: String? = modelToCopy.threadId

            return SSKMessageSenderJobRecord(grdbId: id,
                                             uniqueId: uniqueId,
                                             failureCount: failureCount,
                                             label: label,
                                             sortId: sortId,
                                             status: status,
                                             invisibleMessage: invisibleMessage,
                                             isMediaMessage: isMediaMessage,
                                             messageId: messageId,
                                             removeMessageAfterSending: removeMessageAfterSending,
                                             threadId: threadId)
        }

        if let modelToCopy = self as? SSKMessageDecryptJobRecord {
            assert(type(of: modelToCopy) == SSKMessageDecryptJobRecord.self)
            let uniqueId: String = modelToCopy.uniqueId
            let failureCount: UInt = modelToCopy.failureCount
            let label: String = modelToCopy.label
            let sortId: UInt64 = modelToCopy.sortId
            let status: SSKJobRecordStatus = modelToCopy.status
            let envelopeData: Data? = modelToCopy.envelopeData
            let serverDeliveryTimestamp: UInt64 = modelToCopy.serverDeliveryTimestamp

            return SSKMessageDecryptJobRecord(grdbId: id,
                                              uniqueId: uniqueId,
                                              failureCount: failureCount,
                                              label: label,
                                              sortId: sortId,
                                              status: status,
                                              envelopeData: envelopeData,
                                              serverDeliveryTimestamp: serverDeliveryTimestamp)
        }

        if let modelToCopy = self as? OWSSessionResetJobRecord {
            assert(type(of: modelToCopy) == OWSSessionResetJobRecord.self)
            let uniqueId: String = modelToCopy.uniqueId
            let failureCount: UInt = modelToCopy.failureCount
            let label: String = modelToCopy.label
            let sortId: UInt64 = modelToCopy.sortId
            let status: SSKJobRecordStatus = modelToCopy.status
            let contactThreadId: String = modelToCopy.contactThreadId

            return OWSSessionResetJobRecord(grdbId: id,
                                            uniqueId: uniqueId,
                                            failureCount: failureCount,
                                            label: label,
                                            sortId: sortId,
                                            status: status,
                                            contactThreadId: contactThreadId)
        }

        if let modelToCopy = self as? OWSIncomingGroupSyncJobRecord {
            assert(type(of: modelToCopy) == OWSIncomingGroupSyncJobRecord.self)
            let uniqueId: String = modelToCopy.uniqueId
            let failureCount: UInt = modelToCopy.failureCount
            let label: String = modelToCopy.label
            let sortId: UInt64 = modelToCopy.sortId
            let status: SSKJobRecordStatus = modelToCopy.status
            let attachmentId: String = modelToCopy.attachmentId

            return OWSIncomingGroupSyncJobRecord(grdbId: id,
                                                 uniqueId: uniqueId,
                                                 failureCount: failureCount,
                                                 label: label,
                                                 sortId: sortId,
                                                 status: status,
                                                 attachmentId: attachmentId)
        }

        if let modelToCopy = self as? OWSIncomingContactSyncJobRecord {
            assert(type(of: modelToCopy) == OWSIncomingContactSyncJobRecord.self)
            let uniqueId: String = modelToCopy.uniqueId
            let failureCount: UInt = modelToCopy.failureCount
            let label: String = modelToCopy.label
            let sortId: UInt64 = modelToCopy.sortId
            let status: SSKJobRecordStatus = modelToCopy.status
            let attachmentId: String = modelToCopy.attachmentId

            return OWSIncomingContactSyncJobRecord(grdbId: id,
                                                   uniqueId: uniqueId,
                                                   failureCount: failureCount,
                                                   label: label,
                                                   sortId: sortId,
                                                   status: status,
                                                   attachmentId: attachmentId)
        }

        if let modelToCopy = self as? OWSBroadcastMediaMessageJobRecord {
            assert(type(of: modelToCopy) == OWSBroadcastMediaMessageJobRecord.self)
            let uniqueId: String = modelToCopy.uniqueId
            let failureCount: UInt = modelToCopy.failureCount
            let label: String = modelToCopy.label
            let sortId: UInt64 = modelToCopy.sortId
            let status: SSKJobRecordStatus = modelToCopy.status
            // NOTE: If this generates build errors, you made need to
            // implement DeepCopyable for this type in DeepCopy.swift.
            let attachmentIdMap: [String: [String]] = try DeepCopies.deepCopy(modelToCopy.attachmentIdMap)

            return OWSBroadcastMediaMessageJobRecord(grdbId: id,
                                                     uniqueId: uniqueId,
                                                     failureCount: failureCount,
                                                     label: label,
                                                     sortId: sortId,
                                                     status: status,
                                                     attachmentIdMap: attachmentIdMap)
        }

        do {
            let modelToCopy = self
            assert(type(of: modelToCopy) == SSKJobRecord.self)
            let uniqueId: String = modelToCopy.uniqueId
            let failureCount: UInt = modelToCopy.failureCount
            let label: String = modelToCopy.label
            let sortId: UInt64 = modelToCopy.sortId
            let status: SSKJobRecordStatus = modelToCopy.status

            return SSKJobRecord(grdbId: id,
                                uniqueId: uniqueId,
                                failureCount: failureCount,
                                label: label,
                                sortId: sortId,
                                status: status)
        }

    }
}

// MARK: - Table Metadata

extension SSKJobRecordSerializer {

    // This defines all of the columns used in the table
    // where this model (and any subclasses) are persisted.
    static let idColumn = SDSColumnMetadata(columnName: "id", columnType: .primaryKey)
    static let recordTypeColumn = SDSColumnMetadata(columnName: "recordType", columnType: .int64)
    static let uniqueIdColumn = SDSColumnMetadata(columnName: "uniqueId", columnType: .unicodeString, isUnique: true)
    // Properties
    static let failureCountColumn = SDSColumnMetadata(columnName: "failureCount", columnType: .int64)
    static let labelColumn = SDSColumnMetadata(columnName: "label", columnType: .unicodeString)
    static let statusColumn = SDSColumnMetadata(columnName: "status", columnType: .int)
    static let attachmentIdMapColumn = SDSColumnMetadata(columnName: "attachmentIdMap", columnType: .blob, isOptional: true)
    static let contactThreadIdColumn = SDSColumnMetadata(columnName: "contactThreadId", columnType: .unicodeString, isOptional: true)
    static let envelopeDataColumn = SDSColumnMetadata(columnName: "envelopeData", columnType: .blob, isOptional: true)
    static let invisibleMessageColumn = SDSColumnMetadata(columnName: "invisibleMessage", columnType: .blob, isOptional: true)
    static let messageIdColumn = SDSColumnMetadata(columnName: "messageId", columnType: .unicodeString, isOptional: true)
    static let removeMessageAfterSendingColumn = SDSColumnMetadata(columnName: "removeMessageAfterSending", columnType: .int, isOptional: true)
    static let threadIdColumn = SDSColumnMetadata(columnName: "threadId", columnType: .unicodeString, isOptional: true)
    static let attachmentIdColumn = SDSColumnMetadata(columnName: "attachmentId", columnType: .unicodeString, isOptional: true)
    static let isMediaMessageColumn = SDSColumnMetadata(columnName: "isMediaMessage", columnType: .int, isOptional: true)
    static let serverDeliveryTimestampColumn = SDSColumnMetadata(columnName: "serverDeliveryTimestamp", columnType: .int64, isOptional: true)

    // TODO: We should decide on a naming convention for
    //       tables that store models.
    public static let table = SDSTableMetadata(collection: SSKJobRecord.collection(),
                                               tableName: "model_SSKJobRecord",
                                               columns: [
        idColumn,
        recordTypeColumn,
        uniqueIdColumn,
        failureCountColumn,
        labelColumn,
        statusColumn,
        attachmentIdMapColumn,
        contactThreadIdColumn,
        envelopeDataColumn,
        invisibleMessageColumn,
        messageIdColumn,
        removeMessageAfterSendingColumn,
        threadIdColumn,
        attachmentIdColumn,
        isMediaMessageColumn,
        serverDeliveryTimestampColumn
        ])
}

// MARK: - Save/Remove/Update

@objc
public extension SSKJobRecord {
    func anyInsert(transaction: SDSAnyWriteTransaction) {
        sdsSave(saveMode: .insert, transaction: transaction)
    }

    // Avoid this method whenever feasible.
    //
    // If the record has previously been saved, this method does an overwriting
    // update of the corresponding row, otherwise if it's a new record, this
    // method inserts a new row.
    //
    // For performance, when possible, you should explicitly specify whether
    // you are inserting or updating rather than calling this method.
    func anyUpsert(transaction: SDSAnyWriteTransaction) {
        let isInserting: Bool
        if SSKJobRecord.anyFetch(uniqueId: uniqueId, transaction: transaction) != nil {
            isInserting = false
        } else {
            isInserting = true
        }
        sdsSave(saveMode: isInserting ? .insert : .update, transaction: transaction)
    }

    // This method is used by "updateWith..." methods.
    //
    // This model may be updated from many threads. We don't want to save
    // our local copy (this instance) since it may be out of date.  We also
    // want to avoid re-saving a model that has been deleted.  Therefore, we
    // use "updateWith..." methods to:
    //
    // a) Update a property of this instance.
    // b) If a copy of this model exists in the database, load an up-to-date copy,
    //    and update and save that copy.
    // b) If a copy of this model _DOES NOT_ exist in the database, do _NOT_ save
    //    this local instance.
    //
    // After "updateWith...":
    //
    // a) Any copy of this model in the database will have been updated.
    // b) The local property on this instance will always have been updated.
    // c) Other properties on this instance may be out of date.
    //
    // All mutable properties of this class have been made read-only to
    // prevent accidentally modifying them directly.
    //
    // This isn't a perfect arrangement, but in practice this will prevent
    // data loss and will resolve all known issues.
    func anyUpdate(transaction: SDSAnyWriteTransaction, block: (SSKJobRecord) -> Void) {

        block(self)

        guard let dbCopy = type(of: self).anyFetch(uniqueId: uniqueId,
                                                   transaction: transaction) else {
            return
        }

        // Don't apply the block twice to the same instance.
        // It's at least unnecessary and actually wrong for some blocks.
        // e.g. `block: { $0 in $0.someField++ }`
        if dbCopy !== self {
            block(dbCopy)
        }

        dbCopy.sdsSave(saveMode: .update, transaction: transaction)
    }

    // This method is an alternative to `anyUpdate(transaction:block:)` methods.
    //
    // We should generally use `anyUpdate` to ensure we're not unintentionally
    // clobbering other columns in the database when another concurrent update
    // has occured.
    //
    // There are cases when this doesn't make sense, e.g. when  we know we've
    // just loaded the model in the same transaction. In those cases it is
    // safe and faster to do a "overwriting" update
    func anyOverwritingUpdate(transaction: SDSAnyWriteTransaction) {
        sdsSave(saveMode: .update, transaction: transaction)
    }

    func anyRemove(transaction: SDSAnyWriteTransaction) {
        sdsRemove(transaction: transaction)
    }

    func anyReload(transaction: SDSAnyReadTransaction) {
        anyReload(transaction: transaction, ignoreMissing: false)
    }

    func anyReload(transaction: SDSAnyReadTransaction, ignoreMissing: Bool) {
        guard let latestVersion = type(of: self).anyFetch(uniqueId: uniqueId, transaction: transaction) else {
            if !ignoreMissing {
                owsFailDebug("`latest` was unexpectedly nil")
            }
            return
        }

        setValuesForKeys(latestVersion.dictionaryValue)
    }
}

// MARK: - SSKJobRecordCursor

@objc
public class SSKJobRecordCursor: NSObject {
    private let transaction: GRDBReadTransaction
    private let cursor: RecordCursor<JobRecordRecord>?

    init(transaction: GRDBReadTransaction, cursor: RecordCursor<JobRecordRecord>?) {
        self.transaction = transaction
        self.cursor = cursor
    }

    public func next() throws -> SSKJobRecord? {
        guard let cursor = cursor else {
            return nil
        }
        guard let record = try cursor.next() else {
            return nil
        }
        return try SSKJobRecord.fromRecord(record)
    }

    public func all() throws -> [SSKJobRecord] {
        var result = [SSKJobRecord]()
        while true {
            guard let model = try next() else {
                break
            }
            result.append(model)
        }
        return result
    }
}

// MARK: - Obj-C Fetch

// TODO: We may eventually want to define some combination of:
//
// * fetchCursor, fetchOne, fetchAll, etc. (ala GRDB)
// * Optional "where clause" parameters for filtering.
// * Async flavors with completions.
//
// TODO: I've defined flavors that take a read transaction.
//       Or we might take a "connection" if we end up having that class.
@objc
public extension SSKJobRecord {
    class func grdbFetchCursor(transaction: GRDBReadTransaction) -> SSKJobRecordCursor {
        let database = transaction.database
        do {
            let cursor = try JobRecordRecord.fetchCursor(database)
            return SSKJobRecordCursor(transaction: transaction, cursor: cursor)
        } catch {
            owsFailDebug("Read failed: \(error)")
            return SSKJobRecordCursor(transaction: transaction, cursor: nil)
        }
    }

    // Fetches a single model by "unique id".
    class func anyFetch(uniqueId: String,
                        transaction: SDSAnyReadTransaction) -> SSKJobRecord? {
        assert(uniqueId.count > 0)

        switch transaction.readTransaction {
        case .grdbRead(let grdbTransaction):
            let sql = "SELECT * FROM \(JobRecordRecord.databaseTableName) WHERE \(jobRecordColumn: .uniqueId) = ?"
            return grdbFetchOne(sql: sql, arguments: [uniqueId], transaction: grdbTransaction)
        }
    }

    // Traverses all records.
    // Records are not visited in any particular order.
    class func anyEnumerate(transaction: SDSAnyReadTransaction,
                            block: @escaping (SSKJobRecord, UnsafeMutablePointer<ObjCBool>) -> Void) {
        anyEnumerate(transaction: transaction, batched: false, block: block)
    }

    // Traverses all records.
    // Records are not visited in any particular order.
    class func anyEnumerate(transaction: SDSAnyReadTransaction,
                            batched: Bool = false,
                            block: @escaping (SSKJobRecord, UnsafeMutablePointer<ObjCBool>) -> Void) {
        let batchSize = batched ? Batching.kDefaultBatchSize : 0
        anyEnumerate(transaction: transaction, batchSize: batchSize, block: block)
    }

    // Traverses all records.
    // Records are not visited in any particular order.
    //
    // If batchSize > 0, the enumeration is performed in autoreleased batches.
    class func anyEnumerate(transaction: SDSAnyReadTransaction,
                            batchSize: UInt,
                            block: @escaping (SSKJobRecord, UnsafeMutablePointer<ObjCBool>) -> Void) {
        switch transaction.readTransaction {
        case .grdbRead(let grdbTransaction):
            let cursor = SSKJobRecord.grdbFetchCursor(transaction: grdbTransaction)
            Batching.loop(batchSize: batchSize,
                          loopBlock: { stop in
                                do {
                                    guard let value = try cursor.next() else {
                                        stop.pointee = true
                                        return
                                    }
                                    block(value, stop)
                                } catch let error {
                                    owsFailDebug("Couldn't fetch model: \(error)")
                                }
                              })
        }
    }

    // Traverses all records' unique ids.
    // Records are not visited in any particular order.
    class func anyEnumerateUniqueIds(transaction: SDSAnyReadTransaction,
                                     block: @escaping (String, UnsafeMutablePointer<ObjCBool>) -> Void) {
        anyEnumerateUniqueIds(transaction: transaction, batched: false, block: block)
    }

    // Traverses all records' unique ids.
    // Records are not visited in any particular order.
    class func anyEnumerateUniqueIds(transaction: SDSAnyReadTransaction,
                                     batched: Bool = false,
                                     block: @escaping (String, UnsafeMutablePointer<ObjCBool>) -> Void) {
        let batchSize = batched ? Batching.kDefaultBatchSize : 0
        anyEnumerateUniqueIds(transaction: transaction, batchSize: batchSize, block: block)
    }

    // Traverses all records' unique ids.
    // Records are not visited in any particular order.
    //
    // If batchSize > 0, the enumeration is performed in autoreleased batches.
    class func anyEnumerateUniqueIds(transaction: SDSAnyReadTransaction,
                                     batchSize: UInt,
                                     block: @escaping (String, UnsafeMutablePointer<ObjCBool>) -> Void) {
        switch transaction.readTransaction {
        case .grdbRead(let grdbTransaction):
            grdbEnumerateUniqueIds(transaction: grdbTransaction,
                                   sql: """
                    SELECT \(jobRecordColumn: .uniqueId)
                    FROM \(JobRecordRecord.databaseTableName)
                """,
                batchSize: batchSize,
                block: block)
        }
    }

    // Does not order the results.
    class func anyFetchAll(transaction: SDSAnyReadTransaction) -> [SSKJobRecord] {
        var result = [SSKJobRecord]()
        anyEnumerate(transaction: transaction) { (model, _) in
            result.append(model)
        }
        return result
    }

    // Does not order the results.
    class func anyAllUniqueIds(transaction: SDSAnyReadTransaction) -> [String] {
        var result = [String]()
        anyEnumerateUniqueIds(transaction: transaction) { (uniqueId, _) in
            result.append(uniqueId)
        }
        return result
    }

    class func anyCount(transaction: SDSAnyReadTransaction) -> UInt {
        switch transaction.readTransaction {
        case .grdbRead(let grdbTransaction):
            return JobRecordRecord.ows_fetchCount(grdbTransaction.database)
        }
    }

    // WARNING: Do not use this method for any models which do cleanup
    //          in their anyWillRemove(), anyDidRemove() methods.
    class func anyRemoveAllWithoutInstantation(transaction: SDSAnyWriteTransaction) {
        switch transaction.writeTransaction {
        case .grdbWrite(let grdbTransaction):
            do {
                try JobRecordRecord.deleteAll(grdbTransaction.database)
            } catch {
                owsFailDebug("deleteAll() failed: \(error)")
            }
        }

        if shouldBeIndexedForFTS {
            FullTextSearchFinder.allModelsWereRemoved(collection: collection(), transaction: transaction)
        }
    }

    class func anyRemoveAllWithInstantation(transaction: SDSAnyWriteTransaction) {
        // To avoid mutationDuringEnumerationException, we need
        // to remove the instances outside the enumeration.
        let uniqueIds = anyAllUniqueIds(transaction: transaction)

        var index: Int = 0
        do {
            try Batching.loop(batchSize: Batching.kDefaultBatchSize,
                              loopBlock: { stop in
                                  guard index < uniqueIds.count else {
                                    stop.pointee = true
                                    return
                                  }
                                  let uniqueId = uniqueIds[index]
                                  index = index + 1
                                  guard let instance = anyFetch(uniqueId: uniqueId, transaction: transaction) else {
                                      owsFailDebug("Missing instance.")
                                      return
                                  }
                                  instance.anyRemove(transaction: transaction)
            })
        } catch {
            owsFailDebug("Error: \(error)")
        }

        if shouldBeIndexedForFTS {
            FullTextSearchFinder.allModelsWereRemoved(collection: collection(), transaction: transaction)
        }
    }

    class func anyExists(uniqueId: String,
                        transaction: SDSAnyReadTransaction) -> Bool {
        assert(uniqueId.count > 0)

        switch transaction.readTransaction {
        case .grdbRead(let grdbTransaction):
            let sql = "SELECT EXISTS ( SELECT 1 FROM \(JobRecordRecord.databaseTableName) WHERE \(jobRecordColumn: .uniqueId) = ? )"
            let arguments: StatementArguments = [uniqueId]
            return try! Bool.fetchOne(grdbTransaction.database, sql: sql, arguments: arguments) ?? false
        }
    }
}

// MARK: - Swift Fetch

public extension SSKJobRecord {
    class func grdbFetchCursor(sql: String,
                               arguments: StatementArguments = StatementArguments(),
                               transaction: GRDBReadTransaction) -> SSKJobRecordCursor {
        do {
            let sqlRequest = SQLRequest<Void>(sql: sql, arguments: arguments, cached: true)
            let cursor = try JobRecordRecord.fetchCursor(transaction.database, sqlRequest)
            return SSKJobRecordCursor(transaction: transaction, cursor: cursor)
        } catch {
            Logger.verbose("sql: \(sql)")
            owsFailDebug("Read failed: \(error)")
            return SSKJobRecordCursor(transaction: transaction, cursor: nil)
        }
    }

    class func grdbFetchOne(sql: String,
                            arguments: StatementArguments = StatementArguments(),
                            transaction: GRDBReadTransaction) -> SSKJobRecord? {
        assert(sql.count > 0)

        do {
            let sqlRequest = SQLRequest<Void>(sql: sql, arguments: arguments, cached: true)
            guard let record = try JobRecordRecord.fetchOne(transaction.database, sqlRequest) else {
                return nil
            }

            return try SSKJobRecord.fromRecord(record)
        } catch {
            owsFailDebug("error: \(error)")
            return nil
        }
    }
}

// MARK: - SDSSerializer

// The SDSSerializer protocol specifies how to insert and update the
// row that corresponds to this model.
class SSKJobRecordSerializer: SDSSerializer {

    private let model: SSKJobRecord
    public required init(model: SSKJobRecord) {
        self.model = model
    }

    // MARK: - Record

    func asRecord() throws -> SDSRecord {
        let id: Int64? = model.sortId > 0 ? Int64(model.sortId) : model.grdbId?.int64Value

        let recordType: SDSRecordType = .jobRecord
        let uniqueId: String = model.uniqueId

        // Properties
        let failureCount: UInt = model.failureCount
        let label: String = model.label
        let status: SSKJobRecordStatus = model.status
        let attachmentIdMap: Data? = nil
        let contactThreadId: String? = nil
        let envelopeData: Data? = nil
        let invisibleMessage: Data? = nil
        let messageId: String? = nil
        let removeMessageAfterSending: Bool? = nil
        let threadId: String? = nil
        let attachmentId: String? = nil
        let isMediaMessage: Bool? = nil
        let serverDeliveryTimestamp: UInt64? = nil

        return JobRecordRecord(delegate: model, id: id, recordType: recordType, uniqueId: uniqueId, failureCount: failureCount, label: label, status: status, attachmentIdMap: attachmentIdMap, contactThreadId: contactThreadId, envelopeData: envelopeData, invisibleMessage: invisibleMessage, messageId: messageId, removeMessageAfterSending: removeMessageAfterSending, threadId: threadId, attachmentId: attachmentId, isMediaMessage: isMediaMessage, serverDeliveryTimestamp: serverDeliveryTimestamp)
    }
}

// MARK: - Deep Copy

#if TESTABLE_BUILD
@objc
public extension SSKJobRecord {
    // We're not using this method at the moment,
    // but we might use it for validation of
    // other deep copy methods.
    func deepCopyUsingRecord() throws -> SSKJobRecord {
        guard let record = try asRecord() as? JobRecordRecord else {
            throw OWSAssertionError("Could not convert to record.")
        }
        return try SSKJobRecord.fromRecord(record)
    }
}
#endif
